package com.youkeda.ykd_audio_recorder_example.mediarecorder;


import android.annotation.TargetApi;
import android.media.MediaRecorder;

import android.os.Build;
import android.os.Environment;

import android.support.annotation.RequiresApi;
import android.util.Log;
import android.widget.Button;
import android.widget.ListView;



import java.io.File;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;


/**
 * @author ldm
 * @description 通过MediaRecorder录制声音(基于文件录音，使用方便)并播放
 * @time 2017/2/9 9:02
 */
public class MediaRecorderr {
    private Button start_tv;
    private ListView listView;

    //录音API
    private MediaRecorder mMediaRecorder;
    //录音开始时间与结束时间
    public long startTime, endTime;
    //录音所保存的文件
    private File mAudioFile;
    //文件列表数据
    private List<FileBean> dataList;
    //录音文件保存位置
    public String mFilePath = Environment.getExternalStorageDirectory().getAbsolutePath() + "/audio/";


    public String mSavePath;
    /**
     * @description 开始进行录音
     * @author ldm
     * @time 2017/2/9 9:18
     */
    public void startRecord(String savePath) {

            mSavePath = savePath;



                //播放前释放资源
                releaseRecorder();
                //执行录音操作
                recordOperation();
            }



    /**
     * @description 录音操作
     * @author ldm
     * @time 2017/2/9 9:34
     */
    private void recordOperation() {
        if(mSavePath!=null){
            //创建录音文件,.m4a为MPEG-4音频标准的文件的扩展名
            mAudioFile = new File(mSavePath);
        }else{
            //创建录音文件,.m4a为MPEG-4音频标准的文件的扩展名
            mAudioFile = new File(mFilePath + System.currentTimeMillis() + ".m4a");

        }
        //创建MediaRecorder对象
        mMediaRecorder = new MediaRecorder();
        //创建父文件夹
        mAudioFile.getParentFile().mkdirs();
        try {
            //创建文件
            mAudioFile.createNewFile();
            //配置mMediaRecorder相应参数
            //从麦克风采集声音数据
            mMediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
            //设置保存文件格式为MP4
            mMediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4);
            //设置采样频率,44100是所有安卓设备都支持的频率,频率越高，音质越好，当然文件越大
            mMediaRecorder.setAudioSamplingRate(44100);
            //设置声音数据编码格式,音频通用格式是AAC
            mMediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC);
            //设置编码频率
            mMediaRecorder.setAudioEncodingBitRate(96000);
            //设置录音保存的文件
            mMediaRecorder.setOutputFile(mAudioFile.getAbsolutePath());
            //开始录音
            mMediaRecorder.prepare();
            mMediaRecorder.start();
            //记录开始录音时间
            startTime = System.currentTimeMillis();
        } catch (Exception e) {
            e.printStackTrace();

        }
    }


    /**
     * @description 结束录音操作
     * @author ldm
     * @time 2017/2/9 9:18
     */
    public void stopRecord() {

        //停止录音
        mMediaRecorder.stop();
        //记录停止时间
        endTime = System.currentTimeMillis();
        //录音时间处理，比如只有大于2秒的录音才算成功
        int time = (int) ((endTime - startTime) / 1000);
        if (time >= 3) {
            //录音成功,添加数据
            FileBean bean = new FileBean();
            bean.setFile(mAudioFile);
            bean.setFileLength(time);
            dataList=new ArrayList<>();
            dataList.add(bean);
        } else {
            mAudioFile = null;
        }
        //录音完成释放资源
        releaseRecorder();
    }

    /**
     * @description 翻放录音相关资源
     * @author ldm
     * @time 2017/2/9 9:33
     */
    private void releaseRecorder() {
        if (null != mMediaRecorder) {
            mMediaRecorder.release();
            mMediaRecorder = null;
        }
    }

    /**
     * @description 暂停录音
     * @author ldm
     * @time 2017/2/9 9:33
     */
    @RequiresApi(api = Build.VERSION_CODES.N)
    public void pause(){
        mMediaRecorder.pause();
    }
    /**
     * @description 恢复录音
     * @author ldm
     * @time 2017/2/9 9:33
     */
    public void resume(){
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            mMediaRecorder.resume();
        }


    }
    private int BASE = 1;

    public double getVolumn() {
        double db = 0;// 分贝

        if (mMediaRecorder != null) {
            double ratio = (double)mMediaRecorder.getMaxAmplitude() /BASE;
            if (ratio > 1)
                db = 20 * Math.log10(ratio);

            Log.d("11111","分贝值："+db);
        }
        DecimalFormat d = new DecimalFormat("0.0");
        String vl = d.format(db/100);
        double volumn = Double.valueOf(vl);


        return volumn;
    }


}

